﻿local RezzMon = LibStub("AceAddon-3.0"):NewAddon("RezzMon", "AceConsole-3.0", "AceEvent-3.0", "AceSerializer-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local ResComm = LibStub:GetLibrary("LibResComm-1.0");
local media = LibStub("LibSharedMedia-3.0")
media:Register("statusbar", "Charcoal", "Interface\\Addons\\RezzMon\\textures\\Charcoal")

local raidMembers = {}

local optionsFrame = nil

local function getRaidUnit(name)
	for i,v in ipairs(raidMembers) do
		if v == name then
			return "raid"..i
		end
	end
end

local function debug(msg)
	self = RezzMon
	if self.db.profile.debug then
		self:Print(msg)
	end
end

local BarArranger = {}
BarArranger.bars = {}
function BarArranger:AddBar(bar)
	debug("BarArranger -> Bar added")
	tinsert(self.bars, bar)
	self:Arrange()
end

function BarArranger:GetBar(unit)
	for _,v in ipairs(self.bars) do
		if v.unit == unit then
			debug("BarArranger -> Bar found")
			return v
		end
	end
	debug("BarArranger -> Bar not found")
end

function BarArranger:List()
	return self.bars
end

function BarArranger:RemoveBar(unit)
	for i,v in ipairs(self.bars) do
		if v.unit == unit then
			debug("BarArranger -> Bar removed")
			v:Stop()
			tremove(self.bars, i)
			break
		end
	end
	self:Arrange()
end

function BarArranger:Arrange()
	debug("BarArranger -> Arranging bars")
	for i,v in ipairs(self.bars) do
		if RezzMon.db.profile.growUp then
			v:ClearAllPoints()
			v:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", RezzMon.db.profile.xPos / v:GetEffectiveScale(), (RezzMon.db.profile.yPos + (i - 1) * (RezzMon.db.profile.height + 1)) / v:GetEffectiveScale())
		else
			v:ClearAllPoints()
			v:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", RezzMon.db.profile.xPos / v:GetEffectiveScale(), (RezzMon.db.profile.yPos - (i - 1) * (RezzMon.db.profile.height + 1)) / v:GetEffectiveScale())
		end
	end
end
RezzMon.BarArranger = BarArranger

function RezzMon:InitializeDefaults()
	self.defaults = {
		profile = {
			width = 150,
			height = 12,
			font = "Calibri",
			fontSize = 12,
			texture = "Charcoal",
			xPos = 400,
			yPos = 400,
			growUp = false,
			debug = false
		}
	}
end

function RezzMon:ShowAnchor()
	debug("Options -> Showing Anchor")
	local frame = nil
	if self.anchor then
		frame = self.anchor
	else
		frame = CreateFrame("Frame")
		self.anchor = frame
		frame:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
							  tile = true, tileSize = 16, edgeSize = nil, edgeFile = nil,
							  insets = {left = 0, right = 0, top = 0, bottom = 0} })
		frame:SetBackdropColor(0,0,0,0.8)
		frame.texture = frame:CreateTexture("")
		frame.texture:SetAllPoints(frame)
		frame.Text = frame:CreateFontString(nil, "OVERLAY")
		frame.Text:SetPoint("LEFT", frame, "LEFT", 2, 0)
		frame:SetScript("OnMouseDown", frame.StartMoving)
		frame:SetScript("OnMouseUp", function(self, button)
												self:StopMovingOrSizing()
												if button == "RightButton" then
													RezzMon.db.profile.xPos = self:GetLeft() / self:GetEffectiveScale()
													RezzMon.db.profile.yPos = self:GetBottom() / self:GetEffectiveScale()
													self:Hide()
													if optionsFrame:IsVisible() then RezzMon:OpenConfig() end
												end
											end)
		frame:SetScript("OnEnter", function(self)
												GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
												GameTooltip:SetText("Right Click")
												GameTooltip:AddLine("to save position", 1, 1, 1)
												GameTooltip:Show()
											end)
		frame:SetScript("OnLeave", function(self)
												GameTooltip:Hide()
											end)
		frame:EnableMouse(true)	
		frame:SetMovable(true)
	end
	frame.texture:SetTexture(media:Fetch("statusbar", self.db.profile.texture))
	frame.Text:SetFont(media:Fetch("font", RezzMon.db.profile.font), RezzMon.db.profile.fontSize)
	frame.Text:SetText("Example Text")
	frame:SetWidth(self.db.profile.width)
	frame:SetHeight(self.db.profile.height)
	frame:ClearAllPoints()
	frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", self.db.profile.xPos, self.db.profile.yPos)

	frame:Show()
end

function RezzMon:InitializeOptions()
	self.options = {
		name = "RezzMon",
		type = "group",
		args = {
			postition = {
				name = "Position",
				type = "header",
				order = 10
			},
			x = {
				name = "Left",
				type = "input",
				get = function() return tostring(self.db.profile.xPos) end,
				set = function(_, value) self.db.profile.xPos = value self:ApplySettings() end,
				validate = function(_, value)
									if tonumber(value) then
										return true
									else
										return "Left has to be a number!"
									end
								end,
				order = 11
			},
			y = {
				name = "Bottom",
				type = "input",
				get = function() return tostring(self.db.profile.yPos) end,
				set = function(_, value) self.db.profile.yPos = value self:ApplySettings() end,
				validate = function(_, value)
									if tonumber(value) then
										return true
									else
										return "Bottom has to be a number!"
									end
								end,
				order = 12
			},
			anchor = {
				name = "Show Anchor",
				type = "execute",
				func = function() self:ShowAnchor() end,
				order = 13
			},
			size = {
				name = "Bar Size",
				type = "header",
				order = 20
			},
			width = {
				name = "Width",
				type = "range",
				min = 1,
				max = 300,
				step = 1,
				bigStep = 10,
				get = function() return self.db.profile.width end,
				set = function(_, value)
							self.db.profile.width = value
							self:ApplySettings()
						end,
				order = 21
			},
			height = {
				name = "Height",
				type = "range",
				min = 1,
				max = 30,
				step = 1,
				bigStep = 1,
				get = function() return self.db.profile.height end,
				set = function(_, value)
							self.db.profile.height = value
							self:ApplySettings()
						end,
				order = 22
			},
			display = {
				name = "Display",
				type = "header",
				order = 30
			},
			texture = {
				name = "Texture",
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				get = function() return self.db.profile.texture end,
				set = function(_, value)
							self.db.profile.texture = value
							self:ApplySettings()
						end,
				values = media:HashTable("statusbar"),
				order = 31
			},
			font = {
				name = "Font",
				type = "select",
				dialogControl = 'LSM30_Font',
				get = function() return self.db.profile.font end,
				set = function(_, value)
							self.db.profile.font = value
							self:ApplySettings()
						end,
				values = media:HashTable("font"),
				order = 32
			},
			fontSize = {
				name = "Font Size",
				type = "range",
				min = 1,
				max = 30,
				step = 1,
				get = function() return self.db.profile.fontSize end,
				set = function(_, value)
							self.db.profile.fontSize = value
							self:ApplySettings()
						end,
				order = 33
			},
			growUp = {
				name = "Bars grow up",
				type = "toggle",
				get = function() return self.db.profile.growUp end,
				set = function(_, value) self.db.profile.growUp = value end,
				order = 34
			},
			misc = {
				name = "Misc",
				type = "header",
				order = 40
			},
			debug = {
				name = "Debug",
				type = "toggle",
				get = function() return self.db.profile.debug end,
				set = function(_, value) self.db.profile.debug = value end,
				order = 41
			}
		}
	}
end

function RezzMon:ApplySettings()
	debug("Options -> Applying settings")
	for _,v in pairs(BarArranger:List()) do
		v:ApplySettings()
	end
	if self.anchor and self.anchor:IsShown() then
		self:ShowAnchor()
	end
end

function RezzMon:ResComm_ResStart(event, resser, endTime, target)
	if UnitInRaid("player") then
		local unit = getRaidUnit(resser)
		self:AddBar(unit, target)
	end
end

function RezzMon:ResComm_ResEnd(event, resser, endTime, target)
	if UnitInRaid("player") then
		local unit = getRaidUnit(resser)
		self:RemoveBar(unit)
	end
end

function RezzMon:OnInitialize()
	self:InitializeDefaults()
	self.db = LibStub("AceDB-3.0"):New("RezzMonDB", self.defaults)
	self:InitializeOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RezzMon", self.options)
	self:RegisterChatCommand("RezzMon", self.OpenConfig)
	self:RegisterChatCommand("rezzmon", self.OpenConfig)
	optionsFrame = ACD:AddToBlizOptions("RezzMon")
	ResComm.RegisterCallback(self, "ResComm_ResStart")
	ResComm.RegisterCallback(self, "ResComm_ResEnd")
end

function RezzMon:OnEnable()
	debug("Enabled")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RAID_ROSTER_UPDATE()
end

function RezzMon:OnDisable()
	debug("Disabled")
	for i,_ in pairs(BarArranger:List()) do
		BarArranger:RemoveBar(i)
	end
	self:UnregisterAllEvents()
end

function RezzMon:RAID_ROSTER_UPDATE()
	local self = RezzMon
	if UnitInRaid("player") then
		for i=1,40 do
			local name = GetRaidRosterInfo(i)
			raidMembers[i] = name
		end
	else
		for _,v in ipairs(BarArranger:List()) do
			BarArranger:RemoveBar(v.unit)
		end
	end
end

function RezzMon:OpenConfig()
	InterfaceOptionsFrame_OpenToCategory("RezzMon")
end

function RezzMon:AddBar(unit, target)
	local player = UnitName(unit)
	_, _, _, _, startTime, endTime = UnitCastingInfo(unit)
	if endTime then
		local bar = self.BarTemplate:New(unit, target .. " <- " .. player)
		BarArranger:AddBar(bar)
		bar:SetDuration(endTime - startTime)
		bar:Start()
	end
end

function RezzMon:RemoveBar(unit)
	BarArranger:RemoveBar(unit)
end
